; Game of Memories V1.0
; Stefan Höltgen & Thomas Nückel, 2015
; Sprache: Assembler 2650
;
; Beschreibung:
;
; Dieses Programm ist ein Game of Life nach dem Regelsatz von John Horton Conway.
; Es wurde für den Emulator des Signetics Instructor 50 geschrieben. Die dabei
; verwendete Simulationssoftware ist Winarcadia 22.74. Da der Instructor 50 nur über
; eine rudimentäre Ausgabe verfügt, wurde das Spielfeld für das Game of Life
; in einen 16x16 Byte großen Speicherbereich gelegt, den der Emulator graphisch
; darstellen und so dem Betrachter zugänglich machen kann - wodurch sich der Name des
; Programms erklärt: Game of Memories.
;
; ############################ Technische Erläuterungen #############################
;
;
; Benutzte Variablen:
;
; R0 = Inhaltsregister für geladene Werte (Feldwert)
; R1 = Adressregister zur indexierten Ansteuerung von Adressen in den Feldern
; R2 = Register für Spielarithmetik (Umfeldsumme)
; R3 = Allzweckregister, u. a. zum Zwischenspeichern
; R2 und R3 werden außerdem als Register für die Warteschleife verwendet
;
; Vom Programm benutzte Adressen:
;
; 0000h-00FFh = Adressen für das Spielfeld
; 0E00h-0EFFh = Adressen für das Arbeitsfeld
; 0F00h-0F07h = Adressen für Arithmetik-Memory
; 0F09h-0F0Ah = Adressen für Generations-Zahl (dezimal)
; Kommentar: Für jedes Game of Life sind - solange kein anderer Buffer verwendet
; wird - zwei Felder notwendig. Bei Verwendung von nur einem Feld würden sonst
; Zellen überschrieben werden, die noch für die weitere Analyse notwendig sind.
; Deswegen gibt es zwei Felder: Ein Spiel- und ein Arbeitsfeld.
; 
; Programmstruktur (mit Labeln und Adressen):
;
; INIT (1780h): Initialisierung von Spiel- & Arbeitsfeld
; 	CPA2B   (178Bh): Kopieren ...  
; 	17PROOF (179Dh): ... des Spielfeldes ...
; 	A2BEND  (17A2h): ... ins Arbeitsfeld.
; MAIN (17A5h): Hauptschleife
; 	ANAANF (17ACh): Analyse des Spielfeldes
;		SOND (0180h): Test der Spielfeldecken und -ränder
; 			LOE (02D0h): Sonderfall: Linke obere Ecke
; 			ROE (02F0h): Sonderfall: Rechte obere Ecke
; 			LUE (0310h): Sonderfall: Linke untere Ecke
; 			RUE (0330h): Sonderfall: Rechte untere Ecke
;			RO  (0350h): Sonderfall: Oberer Rand
;			RU  (0380h): Sonderfall: Unterer Rand
;			RR  (03E0h): Sonderfall: Rechter Rand
;			LR  (03B0h): Sonderfall: Linker Rand
;		NORM (0260h): Adress-Arithmetik (Analyse der jeweiligen Nachbarzellen)
; 	REG (0410h): Regeln anwenden
;		REG1 (0413h): Regel 1
;		REG2 (0425h): Regel 2 
;		REG3 (0432h): Regel 3
;	REGEND (043C): Ende der Regelprüfung
; LOOPEND (043Fh): Ende der Hauptschleife
; Subroutinen:
; 	ADDFLD (02B0h): Subroutine: Addition der Umfelder
;	CLRART (0150h): Subroutine: Löschen des Arithmetikfeldes
; 	CLRFLD (0102h): Löschen von Arbeits- und Spielfeld.
; 	SEED   (0120h): Spielfeld ...
;		LOOP  (0126h): ... zufällig ...
;		NOERR (0131h): ... mit Zellen füllen.
;
; ################################ Der Programmcode #################################
;
;ADR	LABEL	MNEMONICS		OPCODES				KOMMENTAR
;
,
; INITIALISIERUNG:
;
1780 	INIT	BCTA,UN CLRFLD	1F 01 02			; Springe zu Subroutine CLRFLD
1783			BCTA,UN SEED	1F 01 20			; Springe zu Subroutine SEED
1786			PPSL 20 		77 20				; Compare-Bit auf logisch stellen
1788			BCTA,UN CPA2B	1F 17 8B			; Springe zu CPA2B
;
; Kopie des Spielfeldes erzeugen:
178B	CPA2B	LODI,R1 00		05 00				; Lade R1 mit 0
178D			LODA,R1 0900	0D 6E 00			; Kopiere den Wert aus der R1-ten
;													  Zelle des Spielfelds nach R0
1790			COMI,R0 15		E4 15				; Ist R0 gleich 15h?
1792			BCFA,2 17PROOF	9C 17 9D			; Nein, dann springe nach 17PROOF
1795			LODI,R0 00		04 01				; Ja, dann lade R0 mit 1 und ...
1797			STRA,R1 6000	CD 60 00			; ... besetze das R1-te Feld auf dem
;													  Arbeitsfeld mit dem Wert aus R0 ...
179A			BCTA,UN A2BEND	1F 17 A2			; ... springe nach A2BEND
179D	17PROOF LODI,R0 01		04 00				; Lade R0 mit 0 und ...
179F			STRA,R1 6000	CD 60 00			; ... besetze das R1-te Feld auf dem
;													  Arbeitsfeld mit dem Wert aus R0
17A2	A2BEND	BIRA,R1 CPA2B	DD 17 8D			; Inkrementiere R1 und springe nach 178D,
;													  solange, bis R1 gleich 0 geworden ist.
;													  Dann weiter.
;
,
; HAUPTSCHLEIFE:
; 
17A5	MAIN	BCTR,UN CLRART	1F 01 50			; Springe zu Subroutine CLRART
17A8			LODI,R0 00		04 00				; Lade R0 mit 0 (Arbeitsfeldadresse)
17AA			LODI,R2 00		06 00				; Lade R2 mit 0 (für Spielarithmetik)
;
; Feld analysieren:
; 
17AC	ANAANF	WRTD,R0			F1					; Analysefortschritt auf LEDs ausgeben		
17AD			BCTA,UN SOND	1F 01 7E			; Springe zur Analyse der Felder
;
; Die normale Spielfeldarithmetik prüft alle Zellen der Moore-Nachbarschaft um die für
; die nächste Generation zu bestimmende Zelle herum - also 8 Zellen, vier orthogonal und vier
; diagonal Anliegende. Das würde aber im Falle einer Eck- oder Randzelle zu falschen
; Ergebnissen führen.
; Darum wird im Folgenden getestet, was für eine Zelle konkret vorliegt, um zu erfahren,
; wieviele Nachbarzellen sie hat und wo diese von ihr aus gesehen liegen.
; Dabei wird zuerst geprüft, ob es sich um eine der 4 Spielfeldecken handelt (die jeweils nur
; 3 Nachbarfelder haben), und wenn nicht, dann, ob die Zelle am Spielfeldrand liegt
; (und damit nur 5 Nachbarn hat). Durch diese Reihenfolge wird auch vermieden, dass
; eine Spielfeldecke fälschlich als Randzelle behandelt wird.
; Handelt es sich auch nicht um eine Randzelle, dann bleibt nur noch, dass es sich um eine
; normale Zelle aus dem Spielfeld handelt.
;
; Auf Spielfeld-Ecken testen:
017E			PPSL 02			77 02				; PPSL umstellen
0180	SOND	COMI,R1 00		E5 00				; Wenn linke obere Ecke (wenn R1 = 00) ...
0182			BCTA,0 LOE		1C 02 D0			; ... dann springe nach LOE
0185			COMI,R1 0F		E5 0F				; Wenn rechte obere Ecke (wenn R1 = 0F) ...
0187			BCTA,0 ROE		1C 02 F0			; ... dann springe nach ROE
018A			COMI,R1 F0		E5 F0				; Wenn linke untere Ecke (wenn R1 = F0) ...
018C			BCTA,0 LUE		1C 03 10			; ... dann springe nach LUE
018F			COMI,R1 FF		E5 FF				; Wenn rechte untere Ecke (wenn R1 = FF) ...
0191			BCTA,0 RUE		1C 03 30			; ... dann springe nach RUE
0194			BCTA,UN			1F 02 3A			; Wenn keine Ecke, springe nach Rand testen.
;
; Wenn es für die hier zu bestimmende Zelle bis jetzt keine Übereinstimmung gegeben hat, dann
; ist sie keine Eckzelle. Sie kann also nur noch eine Rand- oder eine normale Zelle im Feld sein.
; Als nächstes wird darum geprüft, ob es sich bei ihr um eine Randzelle handelt.
;
; Auf Spielfeld-Rand testen:
023A			COMI,R1 10		E5 10				; Wenn oberer Rand (wenn R1 < 10) ...
023C			BCTA,2 RO		1E 03 50			; ... dann springe nach RO
023F			COMI,R1 EF		E5 EF				; Wenn unterer Rand (wenn R1 > EF) ...
0241			BCTA,1 RU		1D 03 80			; ... dann springe nach RU
0244			TMI,R1 0F		F5 0F				; Wenn rechter Rand (wenn R1=xF (xxxx1111)) ...
0246			BCTA,0 RR		1C 03 E0			; ... dann springe nach RR
0249			LODZ,R1			01					; Kopiere den Wert aus R1 nach R0 ...
024A			STRZ,3			C3					; ... Kopiere den Wert aus R0 nach R3 ...
024B			ADDI,R3 0F		87 0F				; ... Addiere 0F zu dem Wert in R3 ...
024D			TMI,R3 0F		F7 0F				; ... ist R3 jetzt gleich 0F, dann linker Rand ...
024F			BCTA,0 LR		1C 03 B0			; ... dann springe nach LR 	
0252			BCTA,UN	NORM	1F 02 60			; Wenn kein Rand, springe nach Normalfeld auslesen.
;
; Wenn es auch bis hierhin keine Übereinstimmung für die zu bestimmende Zelle gegeben hat, dann ist sie
; auch keine Randzelle. Sie kann also nur noch eine normale Zelle im Feld sein. Eine weitere
; Prüfung ist daher nicht notwendig, sondern es kann gleich zur Adress-Arithmetik übergegangen
; werden.
;
,
; ADRESS-ARITHMETIK:
;
; Die arithmetischen Beziehungen zwischen der gerade untersuchten Zelle X und ihren Nachbarzellen
; stellen sich wie folgt dar (innerhalb der Nachbarzellen ist der relative Abstand zu X angegeben;
; außerhalb der relative Abstand der Nachbarzellen zueinander):
; 
;             +1/1h  +1/1h
;             ---->  ---->
;        +------+------+------+
;        | -17/ | -16/ | -15/ |  
; -16/ ^ | -11h | -10h | -Fh  | | +16/
; -10h | +------+------+------+ | +10h
;      | | -1/  |  X   |  +1/ | v
; -16/ ^ | -1h  |      |  +1h | | +16/
; -10h | +------+------+------+ | +10h
;      | | +15/ | +16/ | +17/ | v
;        | +Fh  | +10h | +11h |
;        +------+------+------+
;            <----   <----
;            -1/-1h  -1/-1h
;
; Für die Sonderfälle der Eck- und Randzellen gelten die selben Abstände, nur dass diese
; entsprechend weniger Nachbarn aufweisen. Im Falle der vier Eckzellen jeweils drei Nachbarn, im Falle
; der 56 Randzellen fünf Nachbarn und acht bei den normalen Zellen im Spielfeld.
;
; Auf Basis dieser Überlegungen können nun die Nachbarn der für die nächste Generation
; zu bestimmenden Zelle angesteuert und ihre Werte für die spätere Verwendung gespeichert werden.
; Dabei muss es für jede der 4 Eckzellen, für jeden der 4 Ränder, und für die normalen Zellen
; im Feld eine eigene Adressarithmetik geben, im Ganzen also 9 Adressarithmetiken.
;
; Normalfeld-Adressearithmetik:
; Summe für normale Zelle im Feld erzeugen:
0260	NORM	LODZ,R1			01					; Kopiere den Wert aus R1 nach R0
0261			STRZ,R3			C3					; Kopiere den Wert aus R0 nach R3
0262			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle links)
0264			LODA,R1 0A 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0267			STRA,R0 0F 00	CC 0F 00			; R0 in 0F00h speichern
026A			SUBI,R1 01		A5 10				; R1 = R1 - 10 (für Zelle links oben)
026C			LODA,R1 0A 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
026F			STRA,R0 0F 01	CC 0F 01			; ... in 0F01h speichern
0272			SUBI,R1 11		85 01				; R1 = R1 + 1 (für Zelle oben)
0274			LODA,R1 0A 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0277			STRA,R0 0F 02	CC 0F 02			; ... in 0F02h speichern
027A			SUBI,R1 10		85 01				; R1 = R1 + 1 (für Zelle rechts oben)
027C			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
027F			STRA,R0 0F 03	CC 0F 03			; ... in 0F03h speichern
0282			ADDI,R1 01		85 10				; R1 = R1 + 10 (für Zelle rechts)
0284			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0287			STRA,R0 0F 04	CC 0F 04			; ... in 0F04h speichern
028A			ADDI,R1 11		85 10				; R1 = R1 + 10 (für Zelle rechts unten)
028C			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
028F			STRA,R0 0F 05	CC 0F 05			; ... in 0F05h speichern
0292			ADDI,R1 10		A5 01				; R1 = R1 - 1 (für Zelle unten)
0294			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0297			STRA,R0 0F 06	CC 0F 06			; ... in 0F06h speichern
029A			ADDI,R1 0F		A5 01				; R1 = R1 - 1 (für Zelle links unten)
029C			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
029F			STRA,R0 0F 07	CC 0F 07			; ... in 0F07h speichern
02A2			BCTA,UN ADDFLD	1F 02 B0			; Zur Additions-Subroutine
;
; An dieser Stelle kommt jede Adressarithmetik (auch die noch folgenden) an,
; nachdem sie die Additions-Subroutine aufgerufen hat:
;
; Registerrücktausch - Die Werte in den Registern werden für den nächsten Schritt ausgetauscht
02A5	ENDAA	LODZ,R3			03					; R3 in R0 zurückspeichern
02A6			STRZ,R1			C1					; R0 in R1 zurückspeichern
02A7			BCTA,UN REG 	1F 04 10			; unbedingt zur Regelanwendung
;
; Spielfeldecken-Adressearithmetik
;
; Summe für linke obere Ecke erzeugen:
02D0	LOE		STRZ,R1	 		01					; Kopiere den Wert aus R1 nach R0
02D1			STRZ,R3	 		C3					; Kopiere den Wert aus R0 nach R3
02D2			ADDI,R1 01		85 01				; R1 = R1 + 1 (für Zelle rechts)
02D4			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
02D7			STRA,R0 0F00	CC 0F 00			; R0 in 0F00h sichern
02DA			ADDI,R1 10		85 10				; R1 = R1 + 10 (für Zelle rechts unten)
02DC			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
02DF			STRA,R0 0F01 	CC 0F 01			; R0 in 0F01h sichern
02EB			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle unten)
02E4			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
02E7			STRA,R0 0F02	CC 0F 02			; R0 in 0F02h sichern
02EA			BCTA,UN ADDFLD	1F 02 B0			; Zur Additions-Subroutine
;
; Summe für rechte obere Ecke erzeugen:
02F0	ROE		STRZ,R1	 		01					; Kopiere den Wert aus R1 nach R0
02F1			STRZ,R3	 		C3					; Kopiere den Wert aus R0 nach R3
02F2			ADDI,R1 10		85 10				; R1 = R1 + 10 (für Zelle unten)
02F4			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
02F7			STRA,R0 0F00	CC 0F 00			; R0 in 0F00h sichern
02FA			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle links unten)
02FC			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
02FF			STRA,R0 0F01	CC 0F 01			; R0 in 0F01h sichern
0302			SUBI,R1 10		A5 10				; R1 = R1 - 10 (für Zelle links)
0304			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0307			STRA,R0 0F02	CC 0F 02			; R0 in 0F02h sichern
030A			BCTA,UN ADDFLD	1F 02 B0			; Zur Additions-Subroutine
;
; Summe für linke untere Ecke erzeugen:
0310	LUE		STRZ,R1	 		01					; Kopiere den Wert aus R1 nach R0
0311			STRZ,R3	 		C3					; Kopiere den Wert aus R0 nach R3
0312			SUBI,R1 10		A5 10				; R1 = R1 - 10 (für Zelle oben)
0314			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0317			STRA,R0 0F00	CC 0F 00			; R0 in 0F00h sichern
031A			ADDI,R1 01		85 01				; R1 = R1 + 1 (für Zelle rechts oben)
031C			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
031F			STRA,R0 0F01	CC 0F 01			; R0 in 0F01h sichern
0322			ADDI,R1 10		85 10				; R1 = R1 + 10 (für Zelle rechts)
0324			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0327			STRA,R0 0F02	CC 0F 02			; R0 in 0F02h sichern
032A			BCTA,UN ADDFLD	1F 02 B0			; Zur Additions-Subroutine
;
; Summe für rechte untere Ecke erzeugen:
0330	RUE		STRZ,R1	 		01					; Kopiere den Wert aus R1 nach R0
0331			STRZ,R3	 		C3					; Kopiere den Wert aus R0 nach R3
0332			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle links)
0334			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0337			STRA,R0 0F00	CC 0F 00			; R0 in 0F00h sichern
033A			SUBI,R1 10		A5 10				; R1 = R1 - 10 (für Zelle links oben)
033C			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
033F			STRA,R0 0F01	CC 0F 01			; R0 in 0F01h sichern
0342			ADDI,R1 01		85 01				; R1 = R1 + 1 (für Zelle oben)
0344			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0347			STRA,R0 0F02	CC 0F 02			; R0 in 0F02h sichern
034A			BCTA,UN ADDFLD	1F 02 B0			; Zur Additions-Subroutine
;
; Spielfeldrand-Adressearithmetik:
;
; Summe für oberen Rand erzeugen:
0350	RO		STRZ,R1	 		01					; Kopiere den Wert aus R1 nach R0
0351			STRZ,R3	 		C3					; Kopiere den Wert aus R0 nach R3
0352			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle links)
0354			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0357			STRA,R0 0F 00	CC 0F 00			; R0 in 0F00h speichern
035A			ADDI,R1 02		85 02				; R1 = R1 + 2 (für Zelle rechts)
035C			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
035F			STRA,R0 0F 01	CC 0F 01			; R0 in 0F01h speichern
0362			ADDI,R1 10		85 10				; R1 = R1 + 10 (für Zelle rechts unten)
0364			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0367			STRA,R0 0F 02	CC 0F 02			; R0 in 0F02h speichern
036A			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle unten)
036C			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
036F			STRA,R0 0F 03	CC 0F 03			; R0 in 0F04h speichern
0372			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle links unten)
0374			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0377			STRA,R0 0F 04	CC 0F 04			; R0 in 0F05h speichern
037A			BCTA,UN ADDFLD	1F 02 B0			; Zur Additions-Subroutine
;
; Summe für unteren Rand erzeugen:
0380	RU		STRZ,R1	 		01					; Kopiere den Wert aus R1 nach R0
0381			STRZ,R3	 		C3					; Kopiere den Wert aus R0 nach R3
0382			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle links)
0384			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0387			STRA,R0 0F 00	CC 0F 00			; R0 in 0F00h speichern
038A			SUBI,R1 10		A5 10				; R1 = R1 - 10 (für Zelle links oben)
038C			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
038F			STRA,R0 0F 01	CC 0F 02			; R0 in 0F01h speichern
0392			ADDI,R1 01		85 01				; R1 = R1 + 1 (für Zelle oben)
0394			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0397			STRA,R0 0F 02	CC 0F 02			; R0 in 0F02h speichern
039A			ADDI,R1 01		85 01				; R1 = R1 + 1 (für Zelle rechts oben)
039C			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
039F			STRA,R0 0F 03	CC 0F 03			; R0 in 0F03h speichern
03A2			ADDI,R1 10		85 10				; R1 = R1 + 10 (für Zelle rechts)
03A4			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03A7			STRA,R0 0F 04	CC 0F 04			; R0 in 0F04h speichern
03AA			BCTA,UN ADDFLD	1F 02 B0			; Zur Additions-Subroutine
;
; Summe für linken Rand erzeugen:
03B0	RR		STRZ,R1	 		01					; Kopiere den Wert aus R1 nach R0
03B1			STRZ,R3	 		C3					; Kopiere den Wert aus R0 nach R3
03B2			SUBI,R1 10		A5 10				; R1 = R1 - 10 (für Zelle oben)
03B4			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03B7			STRA,R0 0F 00	CC 0F 00			; R0 in 0F00h speichern
03BA			ADDI,R1 01		85 01				; R1 = R1 + 1 (für Zelle rechts oben)
03BC			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03BF			STRA,R0 0F 01	CC 0F 01			; R0 in 0F01h speichern
03C2			ADDI,R1 10		85 10				; R1 = R1 + 10 (für Zelle rechts)
03C4			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03C7			STRA,R0 0F 02	CC 0F 02			; R0 in 0F02h speichern
03CA			ADDI,R1 10		85 10				; R1 = R1 + 10 (für Zelle rechts unten)
03CC			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03CF			STRA,R0 0F 03	CC 0F 03			; R0 in 0F03h speichern
03D2			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle unten)
03D4			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03D7			STRA,R0 0F 04	CC 0F 04			; R0 in 0F04h speichern
03DA			BCTA,UN ADDFLD	1F 02 B0			; Zur Additions-Subroutine
;
; Summe für rechten Rand erzeugen:
03E0	LR	 	STRZ,R1	 		01					; Kopiere den Wert aus R1 nach R0
93E1			STRZ,R3	 		C3					; Kopiere den Wert aus R0 nach R3
03E2			ADDI,R1 10		85 10				; R1 = R1 + 10 (für Zelle unten)
03E4			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03E7			STRA,R0 0F 00	CC 0F 00			; R0 in 0F00h speichern
03EA			SUBI,R1 01		A5 01				; R1 = R1 - 1 (für Zelle links unten)
03EC			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03EF			STRA,R0 0F 01	CC 0F 01			; R0 in 0F01h speichern
03F2			SUBI,R1 10		A5 10				; R1 = R1 - 10 (für Zelle links)
03F4			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03F7			STRA,R0 0F 02	CC 0F 02			; R0 in 0F02h speichern
03FA			SUBI,R1 10		A5 10				; R1 = R1 - 10 (für Zelle links oben)
03FC			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
03FF			STRA,R0 0F 03	CC 0F 03			; R0 in 0F03h speichern
0402			ADDI,R1 01		85 01				; R1 = R1 + 1 (für Zelle oben)
0404			LODA,R0 06 00	0D 60 00			; Zellwert aus 0000 indiziert über R1 in R0 laden
0407			STRA,R0 0F 04	CC 0F 04			; R0 in 0F04h speichern
040A			BCTA,UN ADDFLD	1F 02 B0			; Zur Additions-Subroutine
;
,
; SPIELREGELN ANWENDEN:
;
; Entscheidend für den Status der Zelle in der nächsten Generation - ob sie tot oder lebendig sein
; wird - ist der momentane Status der Zelle sowie die Anzahl ihrer lebenden Nachbarn. Letztere
; wurde als die Summe der Werte aus den Nachbarzellen durch die Adress-Arithmetik in R2 gespeichert.
; Mit dem momentanen Zustand der Zelle in R0 und diesem Wert in R2 können die Regeln angewandt werden.
; Ausgelesen wird der alte Status einer Zelle jeweils aus dem Arbeitsfeld. Geschrieben wird der neue
; Status derselben Zelle in den "Zwilling" der Zelle auf dem Spielfeld. So wird das erwähnte Problem
; des Überschreibens umgangen.
; Da es mit dem hier verwendeten Assembler nicht möglich ist, gleichzeitig zu testen, ob die Summe
; kleiner als 2 und größer als 3 ist, muss dieser Test in zwei getrennten Regelschritten erfolgen.
; Weiterhin ist zu beachten, dass nur das Arbeitsfeld mit 0en und 1en gefüllt ist. Während also der
; Ist-Zustand einer Zelle auf 0 und 1 hin befragt werden muss, müssen beim Einschreiben des neuen
; Status der Zelle in das Spielfeld nicht 0 und 1, sondern 17 für tot und 15 für lebendig verwendet
; werden.
;
0410	REG		LODA,R1 6A00	0D 60 00			; Hole den Wert der Zelle aus dem Spielfeld ...
;													  ... indziert über R1 in R0.
; Der Wert in R1 bestimmt also, welche Zelle dabei angesteuert wird. R0 enthält danach den momentanten
; Wert der Zelle, deren nächster Zustand bestimmt werden soll.
;
; Regel 1 - WIEDERGEBURT:
; Eine tote Zelle mit 3 lebenden Nachbarn wird neu geboren;
; Wenn ( R0 = 0 und R2 = 3 ) dann R0 = 15; und dieser Wert aus R0 wird dann in den "Zwilling" der Zelle
; auf dem Spielfeld geschrieben:
;
0413	REG1	COMI,R0 00		E4 00				; Ist die Zelle momentan tot? (Ist R0 = 0)
0415			BCFA,0 REG2		9C 04 25			; Wenn sie lebt, dann springe zu Regel 2
0418			COMI,R2 03		E6 03				; Sonst: Ist die Nachbaranzahl gleich 3? (Ist R2 = 3)
041A			BCFA,0 REGEND	9C 04 3C			; Wenn das falsch ist, springe zum Regelprüfung-Ende
041D			LODI,R0 01		04 15				; Sonst: Den Wert für lebende Zelle (15) in R0 laden ...
041F			STRA,R0 0E00	CD 6E 00			; ... und damit die Zelle im Spielfeld wieder beleben
0422			BCTA,UN REGEND	1F 04 3C			; Unbedingt nach Regelprüfung-Ende
;
; Es muss nicht noch einmal geprüft werden, ob die Zelle tot ist. Das hat sich zwangsläufig schon
; ergeben, wenn Regel 1 nicht appliziert wurde, sondern direkt von dort aus nach Regel 2 geprungen
; wurde.
;
; Regel 2 - TOD DURCH ISOLATION:
; Eine lebende Zelle mit weniger als 2 Nachbarn stirbt;
; wenn ( R0 = 1 und R2 < 2 ) dann R0 = 17;  und dieser Wert aus R0 wird dann in den "Zwilling" der Zelle
; auf dem Spielfeld geschrieben:
;
0425	REG2	COMI,R2 02		E6 02				; Ist die Nachbaranzahl kleiner als 2? (Ist R2 < 2)
0427			BCFA,2 REGEND	9E 04 32			; Wenn kleiner, dann nach Regel 3
042A			LODI,R0 17		04 17				; Sonst: Den Wert für tote Zelle (17) in R0 laden ...
042C			STRZ,R1 0E00	CD 6E 00			; ... und damit die Zelle im Spielfeld töten
042F			BCTA,UN REGEND	1F 04 3C			; Unbedingt nach Regelprüfung-Ende 
;
; Regel 3 - TOD DURCH ÜBERBEVÖLKERUNG:
; Eine lebende Zelle mit mehr als 3 Nachbarn stirbt;
; wenn ( R1 = 0 und R2 > 3 ) dann R0 = 17);  und dieser Wert aus R0 wird dann in den "Zwilling" der Zelle
; auf dem Spielfeld geschrieben:
;
0432	REG3	COMI,R2 03		E6 03				; Ist die Nachbaranzahl größer als 3? (Ist R2 > 3)
0434			BCFA,2 REGEND	9D 04 3C			; Wenn größer, dann nach Regelprüfung-Ende
0437			LODI,R1 17		04 17				; Sonst: Den Wert für tote Zelle (17) in R0 laden ...
0439			STRA,R3 0E00	CD 6E 00			; ... und damit die Zelle im Spielfeld töten
;
043C	REGEND	BIRA,R1 ANAANF	DD 17 A5			; Analyseschleifen-Ende; Inkrementiere R1 um global die 
;													  nächste Zelle anzusteuern
;
; LOOP - Lädt die Register mit verschiedenen Werten, die dann dekrementiert werden, für eine Wartschleife
; von 0,5 Sekunden. Andernfalls würde die Geschwindigkeit des Generationswechsels die menschlichen
; Wahrnehmungsschwellen unterlaufen.
; 
043F	LOOPEND	LODI,R2 71		06 71				; Lade 71 nach R2; Äußere Warteschleife
0441			LODI,R3 C2		07 C2				; Lade C2 nach R3; Innere Warteschleife
0443			BDRR,R2 7E		FA 7E				; Dekrementiere R2 bis 0
0445			BDRR,R3 7C		FB 7C				; Dekrementiere R3 bis 0				
0447			BCTA,UN	1786	1F 17 86			; Fertig, zurück in die INITIALISIERUNG
;
;
;
; *********************************************************************************************************
;
; SUBROUTINEN
;
;
; CLRFLD - Löschen des Spielfeldes mit 17, graphisch ausgegeben als " "
;
0102			LODI,R1 00		05 00				; Lade 0 nach R1
0104			LODI,R0 17		04 17				; Lade 15 nach R0
0106			STRA,R1 0E00	CD 6E 00			; Kopiere R0 indexiert über R1 nach 0E00
0109			BIRA,R1 0106	DD 01 06			; Inkeremtiere R1 und springe zurück solange R0 > 0
010C			BCTA,UN INIT	1F 17 83			; zurück nach INIT
;
;
; SEED - Das Spielfeld wird mit Werten gefüllt. Als Random-Seed dient die Anzahl der abgelaufenen
; Clock-Cylcles zum Zeitpunkt des Programmstarts.
;
0120	SEED	CPSL 02			75 02				; COM arithmetisch
0122			LODI,R2 A0		06 FF				; R2=FF	
0124			LODI,R1 FF		05 FF				; Beginn Schleife (255 mal)
0126	LOOP	REDE,R0 73		54 73				; Zufallswert holen
0128			RRL,R0			D0					; nach rechts rotieren
0129			TPSL 04			B5 04				; testet Vorzeichenbit
012B			BCFA NOERR		9C 01 31			; Bei 0 springen
012E			REDE,R3 73		57 73				; noch einen Zufallswert holen
0130			EORZ,R3			22					; R0 XOR R3 -> R0
0131	NOERR	STRZ,R3			C3					; Zufallszahl in R3 ablegen
0132			LODI,R0 15		04 15				; "o" ...
0134			STRA,R3 0900	CF 6E 00			; ... in Feld schreiben
0137			BDRA,R1 LOOP	FD 01 28			; Schleifenende
013A			BCTA,UN 0124	FE 01 24			; Wiederhole Seed
013D			BCTA,UN INIT	1F 17 86			; zurück nach INIT
;
;
; CLRART - Löschen der Arithmetik-Speicherzellen: Da bei der Berechnung der Sonderfelder
; (an den Ecken und Rändern) weniger Nachbarzellen in den Arithmetikspeicher übergeben werden,
; müssen evtl. vorhandene Residuen aus anderen Berechnungen mit 0 gefüllt werden, damit diese 
; die Addition nicht verfälschen.
;
0150	CLRART	LODI,R3 00		07 00				; Lade R3 mit 0
0152			STRA,R3 0F00	CF 0F 00			; Zwischenspeicher löschen
0155			STRA,R3 0F01	CF 0F 01
0158			STRA,R3 0F02	CF 0F 02
015B			STRA,R3 0F03	CF 0F 03
015E			STRA,R3 0F04	CF 0F 04
0161			STRA,R3 0F05	CF 0F 05	
0164			STRA,R3 0F06	CF 0F 06
0167			STRA,R3 0F07	CF 0F 07
016A			STRA,R3 0F08	CF 0F 08
016D			RETC,UN	17A8	1F 17 A8			; Fertig, zurück ins Hauptprogramm
;
; Kommentar: Hier findet sich ein ungefährlicher "Bug": Die Adresse 0F08 wird nie von der ADDFLD
; angesteuert, das Löschen dort und damit die ganze Adresszeile 016A sind unnötig.
;
; ADDFLD - Addition der Umfeld-Speicherzellen-Inhalte: Hier werden die Inhalte der Nachbarzellen
; miteinander addiert, um dann für die Spielregeln ausgewertet zu werden. In den Speicherzellen 
; stehen 15h ("o") oder 17h (" "), die arithmetisch in 1 und 0 umgewandelt werden.
; Die in R2 abgelegte Summe liegt dann zwischen 0 und 8.
;				
02B0	ADDFLD	LODI,R2 00		06 00				; R2 löschen
02B2			ADDA,R2 0F01	8E 0F 00			; (0001h) zu R2 addieren
02B5			ADDA,R2 0F02	8E 0F 01			; (0002h) zu R2 addieren
02B8			ADDA,R2 0F03	8E 0F 02			; (0003h) zu R2 addieren
02BB			ADDA,R2 0F04	8E 0F 03			; (0004h) zu R2 addieren
02BE			ADDA,R2 0F05	8E 0F 04			; (0005h) zu R2 addieren
02C1			ADDA,R2 0F06	8E 0F 05			; (0006h) zu R2 addieren
02C4			ADDA,R2 0F07	8E 0F 06			; (0007h) zu R2 addieren
02C7			ADDA,R2 0F08	8E 0F 07			; (0008h) zu R2 addieren
02CA			BCTA,UN ENDAA	1F 02 A5			; Fertig und zum Registerrücktausch